from stl_sdk.exceptions import InvalidInitParameters
from stl_sdk.atlantis.client import AtlantisClient


class AtlantisClientFlask(AtlantisClient):
    """Cliente Flask para comunicação com Atlantis.
        Ao iniciar o cliente ele irá buscar as variaveis de ambiente
        `ATLANTIS_URL`, `ATLANTIS_CLIENT_ID` e `ATLANTIS_CLIENT_SECRET`

    Exemplo:

        .. code-block:: python

            from stl_sdk.atlantis import AtlantisClientFlask

            atlantis = AtlantisClientFlask()

            def init_app(app):
                atlantis.init_app(app)

    """

    def __init__(self, app=None):
        self.app = app
        if app is not None:
            self.init_app(app)
        super().__init__()

    def init_app(self, app):
        self.atlantis_url = app.config['ATLANTIS_URL']
        self.client_id = app.config['ATLANTIS_CLIENT_ID']
        self.client_secret = app.config.get('ATLANTIS_CLIENT_SECRET')

        if self.atlantis_url is None:
            raise InvalidInitParameters('Missing ATLANTIS_URL on AtlantisClient initialization.')

        if self.client_id is None:
            raise InvalidInitParameters('Missing ATLANTIS_CLIENT_ID on AtlantisClient initialization.')



from stl_sdk.torpedo.client import TorpedoClient
from stl_sdk.exceptions import InvalidInitParameters


class TorpedoClientFlask(TorpedoClient):
    """Cliente Flask para comunicação com Torpedo.
        Ao iniciar o cliente ele irá buscar as variaveis de ambiente
        `TORPEDO_URL` e `TORPEDO_PRODUCT_NAME`

    Exemplo:

        .. code-block:: python

            from stl_sdk.torpedo import TorpedoClientFlask

            torpedo = TorpedoClientFlask()

            def init_app(app):
                torpedo.init_app(app)

    """

    def __init__(self, app=None):
        self.app = app
        if app is not None:
            self.init_app(app)
        super().__init__()

    def init_app(self, app):
        self.torpedo_url = app.config['TORPEDO_URL']
        self.torpedo_product_name = app.config['TORPEDO_PRODUCT_NAME']

        if self.torpedo_url is None:
            raise InvalidInitParameters('Missing TORPEDO_URL on TorpedoClient initialization.')

        if self.torpedo_product_name is None:
            raise InvalidInitParameters('Missing TORPEDO_PRODUCT_NAME on TorpedoClient initialization.')



from requests.exceptions import HTTPError


class CoreHttpError(HTTPError):
    def __init__(self, error):
        self.message = '{}: {}'.format(error.response.status_code, error.response.text)
        self.response = error.response

    def __str__(self):
        return self.message


class InvalidInitParameters(RuntimeError):
    pass



from functools import wraps
from stl_sdk.atlantis.exceptions import AtlantisClientHTTPError


def retry(method):
    """Renova as configurações do openid caso o endpoint retorne 404 e refaz a chamada

    Isso é necessário quando o Atlantis altera o endereço de algum endpoint
    """

    @wraps(method)
    def wrapper(self, *args, **kwargs):
        try:
            return method(self, *args, **kwargs)
        except AtlantisClientHTTPError as error:
            if error.response.status_code == 404:
                self._set_openid_configuration()
                return method(self, *args, **kwargs)
            raise error

    return wrapper
